@@ -219,6 +219,9 @@
 /* The objcode magic header.  */
 #define SCM_OBJCODE_COOKIE                              \
   "GOOF----" SCM_OBJCODE_MACHINE_VERSION_STRING
+#define SCM_OBJCODE_ENDIANNESS_OFFSET 8
+#define SCM_OBJCODE_WORD_SIZE_OFFSET 11
+
 
 #endif  /* SCM__SCM_H */
 
