@@ -119,7 +119,7 @@
 ;;;;
 ;;;; * (pass-if-exception name exception body) will pass if the execution of
 ;;;;   body causes the given exception to be thrown.  If no exception is
-;;;;   thrown, the test fails.  If some other exception is thrown, is is an
+;;;;   thrown, the test fails.  If some other exception is thrown, it is an
 ;;;;   error.
 ;;;; * (expect-fail-exception name exception body) will pass unexpectedly if
 ;;;;   the execution of body causes the given exception to be thrown.  If no
@@ -178,7 +178,7 @@
 ;;;;   ("basic arithmetic" "subtraction"), and
 ;;;;   ("multiplication").
 ;;;;
-;;;; WITH-TEST-PREFIX can be nested.  Each WITH-TEST-PREFIX postpends
+;;;; WITH-TEST-PREFIX can be nested.  Each WITH-TEST-PREFIX appends
 ;;;; a new element to the current prefix:
 ;;;;
 ;;;; (with-test-prefix "arithmetic"
@@ -425,8 +425,7 @@
   (append (current-test-prefix) (list name)))
 
 ;;; A fluid containing the current test prefix, as a list.
-(define prefix-fluid (make-fluid))
-(fluid-set! prefix-fluid '())
+(define prefix-fluid (make-fluid '()))
 (define (current-test-prefix)
   (fluid-ref prefix-fluid))
 
